//////////////////////////////////////////////////////////////////////////////
//
//  GscApi.h
//
//  This is the header file for the General Standards Corp API implementation.
//
#ifdef __cplusplus
extern "C" {
#endif

//
// Macro to export import the DLL routines
//
#ifndef GSCAPI_API
#define GSCAPI_API __declspec(dllimport)
#endif

//
// Macro that determines the maximum number of boards allowed in a system
//
#define	MAXIMUM_NUMBER_OF_BOARDS		10

//
// Macro that defines the number of channels per board
//
#define NUMBER_OF_CHANNELS				4

//
// Macro that defines the maximum number of entries allowed in the DMA queue.
//
#define MAXIMUM_DMA_QUEUE_ITEMS		   100

//
// Macro that defines the maximum clock input to the Zilog Chip.
//
#define MAXIMUM_USC_CLOCK			   20000000

//
// Structure to hold info about the SIO4 boards in the system
//
typedef struct
{
	int		busNumber;
	int		slotNumber;
	int		vendorId;
	int		deviceId;
	char	serialNumber[25];
} GSC_DEVICES_STRUCT;

//
// Structure to hold the current configuration of an HDLC channel
//
typedef struct _GSC_HDLC_CONFIG
{
	int	txUnderRun;
	int txPreamble;
	int txSharedZero;
	int rxLogicalControl;
	int rx16BitControl;
	int rxAddrSearchMode;
	int	packetFraming;
} GSC_HDLC_CONFIG, *PGSC_HDLC_CONFIG;

//
// Structure to hold the current configuration of a BiSync channel
//
typedef struct _GSC_BISYNC_CONFIG
{
	int	txSyncWord;
	int	txUnderRun;
	int txPreamble;
	int	txPreambleLength;
	int txPreamblePattern;
	int txShortSync;
	int	rxSyncWord;
	int rxSyncStrip;
	int rxShortSync;
	int	packetFraming;
} GSC_BISYNC_CONFIG, *PGSC_BISYNC_CONFIG;

//
// Typedef for the callback functions
//
typedef void ((*GSC_CB_FUNCTION)(int boardNumber, int channel, int interrupt));

//
// Tokens for the SIO4 internal register addresses
//
enum GSC_ADDRESSES
{
	// Local Registers
	FW_REVISION_REG    = 0x0000,			// Firmware Revision Register
	BOARD_CONTROL_REG  = 0x0004,			// Board Control Register
	BOARD_STATUS_REG   = 0x0008,			// Board Status Register
	CLOCK_CONTROL_REG  = 0x000c,			// Clock Control Register
	TX_ALMOST_BASE_REG = 0x0010,			// Base value for the Tx Almost registers
	RX_ALMOST_BASE_REG = 0x0014,			// Base value for the Rx Almost registers
	DATA_FIFO_BASE_REG = 0x0018,			// Base value for the Tx/Rx data FIFOs
	CONTROL_STATUS_BASE_REG = 0x001c,		// Base value for the Control/Status registers
	SYNC_CHARACTER_BASE_REG = 0x0050,		// Base value for the Sync Byte Registers
	INTERRUPT_CONTROL_REG   = 0x0060,		// Interrupt Control Register
	INTERRUPT_STATUS_REG    = 0x0064,		// Interrupt Status/Clear Register
	INTERRUPT_EDGE_LEVEL_REG= 0x0068,		// Interrupt Edge/Level Register (RO)
	INTERRUPT_HI_LO_REG		= 0x006c,		// Interrupt High/Low, Rising/Falling register
	PIN_SOURCE_BASE_REG		= 0x0080,		// Base value for the Pin Source Registers
	PIN_STATUS_BASE_REG		= 0x0090,		// Base value for the Pin Status Registers
	POSC_RAM_ADDRESS_REG	= 0x00a0,		// Programmable Osc address register
	POSC_RAM_DATA_REG		= 0x00a4,		// Programmable Osc data register
	POSC_CONTROL_STATUS_REG	= 0x00a8,		// Programmable Osc control/status register
	TX_COUNT_BASE_REG		= 0x00b0,		// -SYNC Tx Counter Reg
	RX_COUNT_BASE_REG		= 0x00c0,		// -SYNC Rx Counter Reg
	FIFO_COUNT_BASE_REG		= 0x00d0,		// Base value for the FIFO Count Registers
	FIFO_SIZE_BASE_REG		= 0x00e0,		// Base value for the FIFO Size Registers
	FEATURES_REG			= 0x00fc,		// Features Register

	// USC/Channel Registers
	USC_CCAR = 0x0000,	// Channel Command/Address Register
	USC_CMR  = 0x0002,	// Channel Mode Register
	USC_CCSR = 0x0004,	// Channel Command/Status Register
	USC_CCR  = 0x0006,	// Channel Control Register
	USC_TMDR = 0x000c,	// Test Mode Data Register
	USC_TMCR = 0x000e,	// Test Mode Control Register
	USC_CMCR = 0x0010,	// Clock Mode Control Register
	USC_HCR  = 0x0012,	// Hardware Configuration Register
	USC_IVR	 = 0x0014,	// Interrupt Vector Register
	USC_IOCR = 0x0016,	// I/O Control Register
	USC_ICR	 = 0x0018,	// Interrupt Control Register
	USC_DCCR = 0x001a,	// Daisy Chain Control Register
	USC_MISR = 0x001c,	// Misc Interrupt Status Register
	USC_SICR = 0x001e,	// Status Interrupt Control Register
	USC_RDR	 = 0x0020,	// Receive Data Register (RO)
	USC_RMR	 = 0x0022,	// Receive Mode Register
	USC_RCSR = 0x0024,	// Receive Command Status Register
	USC_RICR = 0x0026,	// Receive Interrupt Control Register
	USC_RSR	 = 0x0028,	// Receive Sync Register
	USC_RCLR = 0x002a,	// Receive Count Limit Register
	USC_RCCR = 0x002c,	// Receive Character Count Register
	USC_TC0R = 0x002e,	// Time Constant 0 Register
	USC_TDR	 = 0x0030,	// Transmit Data Register (WO)
	USC_TMR	 = 0x0032,	// Transmit Mode Register
	USC_TCSR = 0x0034,	// Transmit Command Status Register
	USC_TICR = 0x0036,	// Transmit Interrupt Control Register
	USC_TSR	 = 0x0038,	// Transmit Sync Register
	USC_TCLR = 0x003a,	// Transmit Count Limit Register
	USC_TCCR = 0x003c,	// Transmit Character Count Register
	USC_TC1R = 0x003e,	// Time Constant 1 Register
};

//
// Tokens for the various interrupt sources
//
enum GSC_INTR_TOKENS
{
	GSC_INTR_SYNC_DETECT			= 0x0001,
	GSC_INTR_USC					= 0x0002,
	GSC_INTR_TX_FIFO_EMPTY			= 0x0004,
	GSC_INTR_TX_FIFO_FULL			= 0x0008,
	GSC_INTR_TX_FIFO_ALMOST_EMPTY	= 0x0010,
	GSC_INTR_RX_FIFO_EMPTY			= 0x0020,
	GSC_INTR_RX_FIFO_FULL			= 0x0040,
	GSC_INTR_RX_FIFO_ALMOST_FULL	= 0x0080,
	GSC_INTR_TX_TRANSFER_COMPLETE	= 0x0100,
	GSC_INTR_RX_TRANSFER_COMPLETE	= 0x0200,
	GSC_INTR_RX_ENVELOPE			= GSC_INTR_SYNC_DETECT,		// -Sync card definition
};


//
// Misc tokens 
//
enum GSC_TOKENS
{
	GSC_MODE_UNINITIALIZED,
	GSC_MODE_ASYNC,
	GSC_MODE_HDLC,
	GSC_MODE_SYNC,
	GSC_MODE_SYNC_ENV,
	GSC_MODE_ISO,
	GSC_MODE_ASYNC_CV,
	GSC_MODE_MONOSYNC,
	GSC_MODE_BISYNC,
	GSC_MODE_TRANS_BISYNC,
	GSC_MODE_NBIF,
	GSC_MODE_802_3,
	GSC_MODE_LAST_MODE,

	GSC_LOOP_NONE,
	GSC_LOOP_EXTERNAL,

	GSC_ABORT,
	GSC_EXTENDED_ABORT,
	GSC_FLAG,
	GSC_CRC_FLAG,
	GSC_ONE_BYTE,
	GSC_ONE_BYTE_CONTROL,
	GSC_EXTENDED_CONTROL,
	GSC_SYN1,
	GSC_SYN0_SYN1,
	GSC_CRC_SYN1,
	GSC_CRC_SYN0_SYN1,
	GSC_PREAMBLE_8BITS,
	GSC_PREAMBLE_16BITS,
	GSC_PREAMBLE_32BITS,
	GSC_PREAMBLE_64BITS,
	GSC_PREAMBLE_ALL_0,
	GSC_PREAMBLE_ALL_1,
	GSC_PREAMBLE_ALL_1_0,
	GSC_PREAMBLE_ALL_0_1,
	GSC_ENABLED,
	GSC_DISABLED,
	GSC_DATA_TRANSMIT,
	GSC_DATA_RECEIVE,

	GSC_PIN_DTE,
	GSC_PIN_DCE,
	GSC_PIN_AUTO,
	GSC_PIN_GPIO,
	GSC_PIN_RX_CLOCK,			// Keep these enums in order
	GSC_PIN_RX_DATA,			//
	GSC_PIN_CTS,				//
	GSC_PIN_DCD,				//
	GSC_PIN_TX_CLOCK,			//
	GSC_PIN_TX_DATA,			//
	GSC_PIN_RTS,				//
	GSC_PIN_AUXCLK,				// Keep these enums in order
	GSC_PIN_RX_ENV,
	GSC_PIN_TX_ENV,

	GSC_ENCODING_NRZ,
	GSC_ENCODING_NRZB,
	GSC_ENCODING_NRZI_MARK,
	GSC_ENCODING_NRZI_SPACE,
	GSC_ENCODING_BIPHASE_MARK,
	GSC_ENCODING_BIPHASE_SPACE,
	GSC_ENCODING_BIPHASE_LEVEL,
	GSC_ENCODING_DIFF_BIPHASE_LEVEL,

	GSC_PROTOCOL_RS422_RS485,
	GSC_PROTOCOL_RS423,
	GSC_PROTOCOL_RS232,
	GSC_PROTOCOL_RS530_1,
	GSC_PROTOCOL_RS530_2,
	GSC_PROTOCOL_V35_1,
	GSC_PROTOCOL_V35_2,
	GSC_PROTOCOL_RS422_RS423_1,
	GSC_PROTOCOL_RS422_RS423_2,

	GSC_TERMINATION_ENABLED,
	GSC_TERMINATION_DISABLED,

	GSC_PARITY_NONE,
	GSC_PARITY_EVEN,
	GSC_PARITY_ODD,
	GSC_PARITY_MARK,
	GSC_PARITY_SPACE,

	GSC_STOP_BITS_0,
	GSC_STOP_BITS_1,
	GSC_STOP_BITS_1_5,
	GSC_STOP_BITS_2,

	GSC_CRC_NONE,
	GSC_CRC_16,
	GSC_CRC_32,
	GSC_CRC_CCITT,
	GSC_CRC_INIT_0,
	GSC_CRC_INIT_1,

	GSC_INTR_RISING_EDGE,
	GSC_INTR_FALLING_EDGE,
	GSC_INTR_HIGH_TRUE,
	GSC_INTR_LOW_TRUE,

	GSC_CLOCK_INTERNAL,
	GSC_CLOCK_EXTERNAL,

	GSC_LSB_FIRST,
	GSC_MSB_FIRST,
};


//
// System Level Routines
//
GSCAPI_API int GscSio4FindBoards( int *boardCount, GSC_DEVICES_STRUCT *results);
GSCAPI_API int GscSio4GetErrorString( int errorCode, char *errorString);

//
// Board Level Routines
//
GSCAPI_API int GscSio4Open( int boardNumber);
GSCAPI_API int GscSio4Close( int boardNumber);
GSCAPI_API int GscSio4GetVersions( int boardNumber, int *libVersion, int *driverVersion, int *fpgaVersion);
GSCAPI_API int GscSio4LocalRegisterRead( int boardNumber, int reg, int *result);
GSCAPI_API int GscSio4LocalRegisterWrite(int boardNumber, int reg, int value);

//
// Channel Level Routines
//
GSCAPI_API int GscSio4ChannelReset(int boardNumber, int channel);
GSCAPI_API int GscSio4ChannelResetRxFifo(int boardNumber, int channel);
GSCAPI_API int GscSio4ChannelResetTxFifo(int boardNumber, int channel);
GSCAPI_API int GscSio4ChannelRegisterRead(int boardNumber, int channel, int reg, int *value);
GSCAPI_API int GscSio4ChannelRegisterWrite(int boardNumber, int channel, int reg, int value);
GSCAPI_API int GscSio4ChannelGetMode(int boardNumber, int channel, int *mode, int *bitRate);
GSCAPI_API int GscSio4ChannelSetMode(int boardNumber, int channel, int mode, int bitRate);
GSCAPI_API int GscSio4ChannelGetDataSize(int boardNumber, int channel, int *dataSize);
GSCAPI_API int GscSio4ChannelSetDataSize(int boardNumber, int channel, int dataSize);
GSCAPI_API int GscSio4ChannelGetGapSize(int boardNumber, int channel, int *gapSize);
GSCAPI_API int GscSio4ChannelSetGapSize(int boardNumber, int channel, int gapSize);
GSCAPI_API int GscSio4ChannelGetMsbLsbOrder(int boardNumber, int channel, int *txOrder, int *rxOrder);
GSCAPI_API int GscSio4ChannelSetMsbLsbOrder(int boardNumber, int channel, int txOrder, int rxOrder);
GSCAPI_API int GscSio4ChannelGetEncoding(int boardNumber, int channel, int *encoding);
GSCAPI_API int GscSio4ChannelSetEncoding(int boardNumber, int channel, int encoding);
GSCAPI_API int GscSio4ChannelGetProtocol(int boardNumber, int channel, int *protocol, int *termination);
GSCAPI_API int GscSio4ChannelSetProtocol(int boardNumber, int channel, int protocol, int termination);
GSCAPI_API int GscSio4ChannelGetParity(int boardNumber, int channel, int *parity);
GSCAPI_API int GscSio4ChannelSetParity(int boardNumber, int channel, int parity);
GSCAPI_API int GscSio4ChannelGetStopBits(int boardNumber, int channel, int *stopBits);
GSCAPI_API int GscSio4ChannelSetStopBits(int boardNumber, int channel, int stopBits);
GSCAPI_API int GscSio4ChannelGetDteDce(int boardNumber, int channel, int *mode);
GSCAPI_API int GscSio4ChannelSetDteDce(int boardNumber, int channel, int mode);
GSCAPI_API int GscSio4ChannelGetTxAlmost(int boardNumber, int channel, int *almostValue);
GSCAPI_API int GscSio4ChannelSetTxAlmost(int boardNumber, int channel, int almostValue);
GSCAPI_API int GscSio4ChannelGetRxAlmost(int boardNumber, int channel, int *almostValue);
GSCAPI_API int GscSio4ChannelSetRxAlmost(int boardNumber, int channel, int almostValue);
GSCAPI_API int GscSio4ChannelGetLoopBack( int boardNumber, int channel, int *loopMode);
GSCAPI_API int GscSio4ChannelSetLoopBack( int boardNumber, int channel, int loopMode);
GSCAPI_API int GscSio4ChannelSetClock( int boardNumber, int channel, int frequency);
GSCAPI_API int GscSio4ChannelSetClockSource( int boardNumber, int channel, int txSource, int rxSource);
GSCAPI_API int GscSio4ChannelCheckForData(int boardNumber, int channel, char *buffer, int *count);
GSCAPI_API int GscSio4ChannelReceiveData(int boardNumber, int channel, char *buffer, int count, int *id);
GSCAPI_API int GscSio4ChannelReceiveDataAndWait(int boardNumber, int channel, char *buffer, int count, int timeout, int *bytesTransferred);
GSCAPI_API int GscSio4ChannelTransmitData(int boardNumber, int channel, char *buffer, int count, int *id);
GSCAPI_API int GscSio4ChannelTransmitDataAndWait(int boardNumber, int channel, char *buffer, int count, int timeout, int *bytesTransferred);
GSCAPI_API int GscSio4ChannelFifoCounts( int boardNumber, int channel, int *counts);
GSCAPI_API int GscSio4ChannelFifoSizes( int boardNumber, int channel, int *sizes);
GSCAPI_API int GscSio4ChannelQueryTransfer(int boardNumber, int channel, int *stat, int id);
GSCAPI_API int GscSio4ChannelWaitForTransfer(int boardNumber, int channel, int timeout, int id, int *bytesTransferred);
GSCAPI_API int GscSio4ChannelWaitForTransfer_A(int boardNumber, int channel, int timeout, int id, int *bytesTransferred);
GSCAPI_API int GscSio4ChannelFlushTransfer( int boardNumber, int channel, int id);
GSCAPI_API int GscSio4ChannelRemoveTransfer( int boardNumber, int channel, int id, int *bytesTransferred);
GSCAPI_API int GscSio4ChannelReceiverControl( int boardNumber, int channel, int rxMode);
GSCAPI_API int GscSio4ChannelTransmitterControl( int boardNumber, int channel, int txMode);
GSCAPI_API int GscSio4ChannelRegisterInterrupt( int boardNumber, int channel, int interrupt, int type, GSC_CB_FUNCTION function);

GSCAPI_API int GscSio4ChannelGetPinMode( int boardNumber, int channel, int pinName, int *mode);
GSCAPI_API int GscSio4ChannelSetPinMode( int boardNumber, int channel, int pinName, int mode);
GSCAPI_API int GscSio4ChannelGetPinValue( int boardNumber, int channel, int pinName, int *value);
GSCAPI_API int GscSio4ChannelSetPinValue( int boardNumber, int channel, int pinName, int value);

//
// Protocol Level Routines
//
GSCAPI_API int GscSio4ChannelGetHdlcConfig(int boardNumber, int channel, PGSC_HDLC_CONFIG config);
GSCAPI_API int GscSio4ChannelSetHdlcConfig(int boardNumber, int channel, GSC_HDLC_CONFIG config);
GSCAPI_API int GscSio4ChannelGetHdlcCrcMode(int boardNumber, int channel, int *crcMode, int *initialValue);
GSCAPI_API int GscSio4ChannelSetHdlcCrcMode(int boardNumber, int channel, int crcMode, int initialValue);

GSCAPI_API int GscSio4ChannelGetBiSyncConfig(int boardNumber, int channel, PGSC_BISYNC_CONFIG config);
GSCAPI_API int GscSio4ChannelSetBiSyncConfig(int boardNumber, int channel, GSC_BISYNC_CONFIG config);
GSCAPI_API int GscSio4ChannelGetBiSyncPattern( int boardNumber, int channel, int *TxSync, int *RxSync);
GSCAPI_API int GscSio4ChannelSetBiSyncPattern( int boardNumber, int channel, int TxSync, int RxSync);
GSCAPI_API int GscSio4ChannelGetBiSyncTxUnderrun( int boardNumber, int channel, int *txUnderrun);
GSCAPI_API int GscSio4ChannelSetBiSyncTxUnderrun( int boardNumber, int channel, int txUnderrun);
GSCAPI_API int GscSio4ChannelGetBiSyncTxPreamble( int boardNumber, int channel, int *txPreamble);
GSCAPI_API int GscSio4ChannelSetBiSyncTxPreamble( int boardNumber, int channel, int txPreamble);
GSCAPI_API int GscSio4ChannelGetBiSyncTxShortSync( int boardNumber, int channel, int *txShortSync);
GSCAPI_API int GscSio4ChannelSetBiSyncTxShortSync( int boardNumber, int channel, int txShortSync);
GSCAPI_API int GscSio4ChannelGetBiSyncRxSyncStrip( int boardNumber, int channel, int *rxSyncStrip);
GSCAPI_API int GscSio4ChannelSetBiSyncRxSyncStrip( int boardNumber, int channel, int rxSyncStrip);
GSCAPI_API int GscSio4ChannelGetBiSyncRxShortSync( int boardNumber, int channel, int *rxShortSync);
GSCAPI_API int GscSio4ChannelSetBiSyncRxShortSync( int boardNumber, int channel, int rxShortSync);
GSCAPI_API int GscSio4ChannelGetBiSyncTxPreambleLength( int boardNumber, int channel, int *txPreambleLength);
GSCAPI_API int GscSio4ChannelSetBiSyncTxPreambleLength( int boardNumber, int channel, int txPreambleLength);
GSCAPI_API int GscSio4ChannelGetBiSyncTxPreamblePattern( int boardNumber, int channel, int *txPreamblePattern);
GSCAPI_API int GscSio4ChannelSetBiSyncTxPreamblePattern( int boardNumber, int channel, int txPreamblePattern);


//GSCAPI_API int GscSio4ChannelGetRxAlmostEmpty(int boardNumber, int channel, int *almostValue);
//GSCAPI_API int GscSio4ChannelGetRxAlmostFull(int boardNumber, int channel, int *almostValue);
//GSCAPI_API int GscSio4ChannelGetTxAlmostEmpty(int boardNumber, int channel, int *almostValue);
//GSCAPI_API int GscSio4ChannelGetTxAlmostFull(int boardNumber, int channel, int *almostValue);

#ifdef __cplusplus
}
#endif
